@@ -976,11 +976,11 @@ static int APR_THREAD_FUNC regfnServerSupportFunction(isapi_cid    *cid,
             return 0;
         }
 
-        len = (apr_uint32_t)strlen(r->filename);
+        len = (apr_uint32_t)strlen(subreq->filename);
 
         if ((subreq->finfo.filetype == APR_DIR)
               && (!subreq->path_info)
-              && (file[len - 1] != '/'))
+              && (subreq->filename[len - 1] != '/'))
             file = apr_pstrcat(cid->r->pool, subreq->filename, "/", NULL);
         else
             file = apr_pstrcat(cid->r->pool, subreq->filename,
