@@ -69,6 +69,7 @@ apr_status_t ap_http_chunk_filter(ap_filter_t *f, apr_bucket_brigade *b)
         {
             if (APR_BUCKET_IS_EOS(e)) {
                 /* there shouldn't be anything after the eos */
+                ap_remove_output_filter(f);
                 eos = e;
                 break;
             }
@@ -186,11 +187,11 @@ apr_status_t ap_http_chunk_filter(ap_filter_t *f, apr_bucket_brigade *b)
 
         /* pass the brigade to the next filter. */
         rv = ap_pass_brigade(f->next, b);
+        apr_brigade_cleanup(b);
         if (rv != APR_SUCCESS || eos != NULL) {
             return rv;
         }
         tmp = b;
-        apr_brigade_cleanup(tmp);
     }
     return APR_SUCCESS;
 }
