@@ -25,7 +25,7 @@
 
 static int TestOptionalFn(const char *szStr)
 {
-    ap_log_error(APLOG_MARK,APLOG_ERR,OK,NULL,
+    ap_log_error(APLOG_MARK,APLOG_ERR,OK,NULL, APLOGNO(01871)
                  "Optional function test said: %s",szStr);
 
     return OK;
@@ -36,7 +36,7 @@ static void ExportRegisterHooks(apr_pool_t *p)
     APR_REGISTER_OPTIONAL_FN(TestOptionalFn);
 }
 
-module AP_MODULE_DECLARE_DATA optional_fn_export_module=
+AP_DECLARE_MODULE(optional_fn_export) =
 {
     STANDARD20_MODULE_STUFF,
     NULL,
